"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var rxjs_1 = require('rxjs');
function pausableBuffer(isPausedObservable, closeBuffer) {
    var source = this;
    return source.lift(new PausableBufferOperator(isPausedObservable, closeBuffer));
}
exports.pausableBuffer = pausableBuffer;
var PausableBufferOperator = (function () {
    function PausableBufferOperator(isPausedObservable, closeBuffer) {
        this.isPausedObservable = isPausedObservable;
        this.closeBuffer = closeBuffer;
    }
    PausableBufferOperator.prototype.call = function (subscriber, source) {
        return source._subscribe(new PausableBufferSubscriber(subscriber, this.isPausedObservable, this.closeBuffer));
    };
    return PausableBufferOperator;
}());
var PausableBufferSubscriber = (function (_super) {
    __extends(PausableBufferSubscriber, _super);
    function PausableBufferSubscriber(destination, isPausedObservable, closeBuffer) {
        var _this = this;
        _super.call(this, destination);
        this.closeBuffer = closeBuffer;
        if (this.closeBuffer == null) {
            this.closeBuffer = (function (x) { return x; });
        }
        this.isPausedSubject = new rxjs_1.BehaviorSubject(false);
        this.buffer = [];
        this.add(isPausedObservable
            .map(function (x) { return x === true; })
            .distinctUntilChanged()
            .subscribe(this.isPausedSubject));
        this.add(this.isPausedSubject
            .subscribe(function (x) { return _this.onPauseChanged(); }));
    }
    PausableBufferSubscriber.prototype.onPauseChanged = function () {
        if (this.isPausedSubject.value === false) {
            this.flushBuffer();
        }
    };
    PausableBufferSubscriber.prototype.flushBuffer = function () {
        var _this = this;
        if (this.buffer.length > 0) {
            var buffer = this.buffer;
            this.buffer = [];
            this
                .closeBuffer(buffer)
                .forEach(function (x) { return _super.prototype._next.call(_this, x); });
        }
    };
    PausableBufferSubscriber.prototype._next = function (value) {
        if (this.isPausedSubject.value === true) {
            this.buffer.push(value);
        }
        else {
            _super.prototype._next.call(this, value);
        }
    };
    PausableBufferSubscriber.prototype._complete = function () {
        this.flushBuffer();
        _super.prototype._complete.call(this);
    };
    return PausableBufferSubscriber;
}(rxjs_1.Subscriber));
