"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var rxjs_1 = require('rxjs');
var ReactiveApp_1 = require('./ReactiveApp');
var SubjectScheduler_1 = require('./SubjectScheduler');
function dedup(batch) {
    // TODO: implement
    return batch;
}
var ReactiveState = (function (_super) {
    __extends(ReactiveState, _super);
    function ReactiveState(errorScheduler) {
        var _this = this;
        _super.call(this);
        this.changeNotificationsSuppressed = 0;
        this.changeNotificationsDelayed = 0;
        this.startDelayNotificationsSubject = new rxjs_1.Subject();
        this.changingSubject = new rxjs_1.Subject();
        this.changedSubject = new rxjs_1.Subject();
        this.thrownErrorsHandler = new SubjectScheduler_1.SubjectScheduler(errorScheduler, ReactiveApp_1.ReactiveApp.defaultErrorHandler);
        this.add(this.startDelayNotificationsSubject);
        this.add(this.changingSubject);
        this.add(this.changedSubject);
        this.add(this.thrownErrorsHandler);
        this.changingObservable = this.changingSubject
            .pausableBuffer(this.startDelayNotificationsSubject
            .map(function () { return _this.areChangeNotificationsDelayed() === true; }), function (x) { return dedup(x); })
            .publish()
            .refCount();
        this.changedObservable = this.changedSubject
            .pausableBuffer(this.startDelayNotificationsSubject
            .map(function () { return _this.areChangeNotificationsDelayed() === true; }), function (x) { return dedup(x); })
            .publish()
            .refCount();
    }
    ReactiveState.prototype.notifyPropertyChanging = function (changing) {
        if (this.areChangeNotificationsEnabled() === false) {
            return;
        }
        this.notifyObservable(changing(), this.changingSubject);
    };
    ReactiveState.prototype.notifyPropertyChanged = function (changed) {
        if (this.areChangeNotificationsEnabled() === false) {
            return;
        }
        this.notifyObservable(changed(), this.changedSubject);
    };
    ReactiveState.prototype.notifyObservable = function (change, subject) {
        try {
            subject.next(change);
        }
        catch (err) {
            this.thrownErrorsHandler.next(err);
        }
    };
    Object.defineProperty(ReactiveState.prototype, "changing", {
        get: function () {
            return this.changingObservable;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ReactiveState.prototype, "changed", {
        get: function () {
            return this.changedObservable;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ReactiveState.prototype, "thrownErrors", {
        get: function () {
            return this.thrownErrorsHandler
                .getScheduledObservable();
        },
        enumerable: true,
        configurable: true
    });
    ReactiveState.prototype.areChangeNotificationsEnabled = function () {
        return this.changeNotificationsSuppressed === 0;
    };
    ReactiveState.prototype.areChangeNotificationsDelayed = function () {
        return this.changeNotificationsDelayed > 0;
    };
    // this function stops notifications from being queued until it is
    // unsubscribed
    ReactiveState.prototype.suppressChangeNotifications = function () {
        var _this = this;
        ++this.changeNotificationsSuppressed;
        return new rxjs_1.Subscription(function () {
            --_this.changeNotificationsSuppressed;
        });
    };
    // this function stops queued notifications from being processed until
    // it is unsubscribed (but new notifications can still be queued)
    ReactiveState.prototype.delayChangeNotifications = function () {
        var _this = this;
        ++this.changeNotificationsDelayed;
        if (this.changeNotificationsDelayed === 1) {
            this.startDelayNotificationsSubject.next(null);
        }
        return new rxjs_1.Subscription(function () {
            --_this.changeNotificationsDelayed;
            if (_this.changeNotificationsDelayed === 0) {
                _this.startDelayNotificationsSubject.next(null);
            }
        });
    };
    return ReactiveState;
}(rxjs_1.Subscription));
exports.ReactiveState = ReactiveState;
