"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var rxjs_1 = require('rxjs');
var ReactiveApp_1 = require('./ReactiveApp');
var SubjectScheduler = (function (_super) {
    __extends(SubjectScheduler, _super);
    function SubjectScheduler(scheduler, defaultObserverOrNext, subject) {
        if (subject === void 0) { subject = new rxjs_1.Subject(); }
        _super.call(this);
        this.scheduler = scheduler;
        this.defaultObserverOrNext = defaultObserverOrNext;
        this.subject = subject;
        this.observerRefCount = 0;
        this.add(this.subject);
        if (this.defaultObserverOrNext != null) {
            this.defaultObserverSub = this.getScheduledObservable()
                .subscribe(defaultObserverOrNext, ReactiveApp_1.ReactiveApp.defaultErrorHandler.next);
        }
    }
    SubjectScheduler.prototype.next = function (value) {
        this.subject.next(value);
    };
    SubjectScheduler.prototype.error = function (err) {
        this.subject.error(err);
    };
    SubjectScheduler.prototype.complete = function () {
        this.subject.complete();
    };
    SubjectScheduler.prototype.getScheduledObservable = function () {
        return this.scheduler == null ?
            this.subject.asObservable() :
            this.subject.observeOn(this.scheduler);
    };
    SubjectScheduler.prototype.subscribe = function (observerOrNext, error, complete) {
        var _this = this;
        if (error === void 0) { error = ReactiveApp_1.ReactiveApp.defaultErrorHandler.next; }
        if (this.defaultObserverSub) {
            this.defaultObserverSub.unsubscribe();
            this.defaultObserverSub = undefined;
        }
        ++this.observerRefCount;
        var sub = this.getScheduledObservable()
            .subscribe(observerOrNext, error, complete);
        sub.add(new rxjs_1.Subscription(function () {
            if (--_this.observerRefCount <= 0 && _this.defaultObserverOrNext != null) {
                _this.observerRefCount = 0;
                _this.defaultObserverSub = _this.getScheduledObservable()
                    .subscribe(_this.defaultObserverOrNext, ReactiveApp_1.ReactiveApp.defaultErrorHandler.next);
            }
        }));
        return sub;
    };
    return SubjectScheduler;
}(rxjs_1.Subscription));
exports.SubjectScheduler = SubjectScheduler;
