import { Subscriber, BehaviorSubject } from 'rxjs';
export function pausableBuffer(isPausedObservable, closeBuffer) {
    const source = this;
    return source.lift(new PausableBufferOperator(isPausedObservable, closeBuffer));
}
class PausableBufferOperator {
    constructor(isPausedObservable, closeBuffer) {
        this.isPausedObservable = isPausedObservable;
        this.closeBuffer = closeBuffer;
    }
    call(subscriber, source) {
        return source._subscribe(new PausableBufferSubscriber(subscriber, this.isPausedObservable, this.closeBuffer));
    }
}
class PausableBufferSubscriber extends Subscriber {
    constructor(destination, isPausedObservable, closeBuffer) {
        super(destination);
        this.closeBuffer = closeBuffer;
        if (this.closeBuffer == null) {
            this.closeBuffer = (x => x);
        }
        this.isPausedSubject = new BehaviorSubject(false);
        this.buffer = [];
        this.add(isPausedObservable
            .map(x => x === true)
            .distinctUntilChanged()
            .subscribe(this.isPausedSubject));
        this.add(this.isPausedSubject
            .subscribe(x => this.onPauseChanged()));
    }
    onPauseChanged() {
        if (this.isPausedSubject.value === false) {
            this.flushBuffer();
        }
    }
    flushBuffer() {
        if (this.buffer.length > 0) {
            const buffer = this.buffer;
            this.buffer = [];
            this
                .closeBuffer(buffer)
                .forEach(x => super._next(x));
        }
    }
    _next(value) {
        if (this.isPausedSubject.value === true) {
            this.buffer.push(value);
        }
        else {
            super._next(value);
        }
    }
    _complete() {
        this.flushBuffer();
        super._complete();
    }
}
