import { ReactiveState } from './ReactiveState';
import { ReactiveEvent } from './ReactiveEvent';
import { ReactiveStreamProperty, ReactiveValueProperty } from './ReactiveProperty';
export class ReactiveObject extends ReactiveState {
    constructor(errorScheduler) {
        super(errorScheduler);
        this.properties = [];
    }
    getPropertyName(prop) {
        if (prop.name == null) {
            this.resolvePropertyNames();
        }
        return prop.name;
    }
    resolvePropertyNames() {
        Object
            .getOwnPropertyNames(this)
            .map(name => ({ name, prop: this[name] }))
            .filter(x => x.prop != null && x.prop.owner === this && x.prop.name == null)
            .forEach(x => {
            x.prop.name = x.name;
        });
    }
    registerProperty(prop) {
        this.add(prop.changing
            .subscribe(x => {
            this.notifyPropertyChanging(() => new ReactiveEvent(this, {
                property: x.source,
                propertyName: this.getPropertyName(x.source),
            }));
        }, this.thrownErrorsHandler.next));
        this.add(prop.changed
            .subscribe(x => {
            this.notifyPropertyChanged(() => new ReactiveEvent(this, {
                property: x.source,
                propertyName: this.getPropertyName(x.source),
            }));
        }, this.thrownErrorsHandler.next));
        this.properties.push(prop);
    }
    propertyFrom(source, initialValue, errorScheduler) {
        const prop = new ReactiveStreamProperty(source, this, initialValue, errorScheduler);
        this.registerProperty(prop);
        return prop;
    }
    property(initialValue) {
        const prop = new ReactiveValueProperty(this, initialValue);
        this.registerProperty(prop);
        return prop;
    }
}
