import { Scheduler as Schedulers } from 'rxjs';
import { SubjectScheduler } from './SubjectScheduler';
import { ReactiveEvent } from './ReactiveEvent';
import { ReactiveState } from './ReactiveState';
export class ReactiveProperty extends ReactiveState {
    constructor(owner, initialValue, errorScheduler) {
        super(errorScheduler);
        this.owner = owner;
        this.currentValue = initialValue;
    }
    initialize(source) {
        this.add(source
            .distinctUntilChanged()
            .subscribe(x => {
            this.notifyPropertyChanging(() => new ReactiveEvent(this, x));
            this.currentValue = x;
            this.notifyPropertyChanged(() => new ReactiveEvent(this, this.currentValue));
        }, this.thrownErrorsHandler.next));
    }
    set name(propertyName) {
        if (this.propertyName != null) {
            throw 'Manually setting a reactive property name is not allowed';
        }
        this.propertyName = propertyName;
    }
    get name() {
        return this.propertyName;
    }
    get value() {
        return this.currentValue;
    }
}
export class ReactiveStreamProperty extends ReactiveProperty {
    constructor(source, owner, initialValue, errorScheduler) {
        super(owner, initialValue, errorScheduler);
        this.source = source;
        this.initialize(source);
    }
}
export class ReactiveValueProperty extends ReactiveProperty {
    constructor(owner, initialValue, scheduler = Schedulers.queue, errorScheduler) {
        super(owner, initialValue, errorScheduler);
        this.valueHandler = new SubjectScheduler(scheduler);
        this.add(this.valueHandler);
        this.initialize(this.valueHandler.getScheduledObservable());
    }
    get source() {
        return this.valueHandler.getScheduledObservable();
    }
    get value() {
        return this.currentValue;
    }
    set value(value) {
        this.valueHandler.next(value);
    }
}
