import { Subscription, Subject } from 'rxjs';
import { ReactiveApp } from './ReactiveApp';
import { SubjectScheduler } from './SubjectScheduler';
function dedup(batch) {
    // TODO: implement
    return batch;
}
export class ReactiveState extends Subscription {
    constructor(errorScheduler) {
        super();
        this.changeNotificationsSuppressed = 0;
        this.changeNotificationsDelayed = 0;
        this.startDelayNotificationsSubject = new Subject();
        this.changingSubject = new Subject();
        this.changedSubject = new Subject();
        this.thrownErrorsHandler = new SubjectScheduler(errorScheduler, ReactiveApp.defaultErrorHandler);
        this.add(this.startDelayNotificationsSubject);
        this.add(this.changingSubject);
        this.add(this.changedSubject);
        this.add(this.thrownErrorsHandler);
        this.changingObservable = this.changingSubject
            .pausableBuffer(this.startDelayNotificationsSubject
            .map(() => this.areChangeNotificationsDelayed() === true), x => dedup(x))
            .publish()
            .refCount();
        this.changedObservable = this.changedSubject
            .pausableBuffer(this.startDelayNotificationsSubject
            .map(() => this.areChangeNotificationsDelayed() === true), x => dedup(x))
            .publish()
            .refCount();
    }
    notifyPropertyChanging(changing) {
        if (this.areChangeNotificationsEnabled() === false) {
            return;
        }
        this.notifyObservable(changing(), this.changingSubject);
    }
    notifyPropertyChanged(changed) {
        if (this.areChangeNotificationsEnabled() === false) {
            return;
        }
        this.notifyObservable(changed(), this.changedSubject);
    }
    notifyObservable(change, subject) {
        try {
            subject.next(change);
        }
        catch (err) {
            this.thrownErrorsHandler.next(err);
        }
    }
    get changing() {
        return this.changingObservable;
    }
    get changed() {
        return this.changedObservable;
    }
    get thrownErrors() {
        return this.thrownErrorsHandler
            .getScheduledObservable();
    }
    areChangeNotificationsEnabled() {
        return this.changeNotificationsSuppressed === 0;
    }
    areChangeNotificationsDelayed() {
        return this.changeNotificationsDelayed > 0;
    }
    // this function stops notifications from being queued until it is
    // unsubscribed
    suppressChangeNotifications() {
        ++this.changeNotificationsSuppressed;
        return new Subscription(() => {
            --this.changeNotificationsSuppressed;
        });
    }
    // this function stops queued notifications from being processed until
    // it is unsubscribed (but new notifications can still be queued)
    delayChangeNotifications() {
        ++this.changeNotificationsDelayed;
        if (this.changeNotificationsDelayed === 1) {
            this.startDelayNotificationsSubject.next(null);
        }
        return new Subscription(() => {
            --this.changeNotificationsDelayed;
            if (this.changeNotificationsDelayed === 0) {
                this.startDelayNotificationsSubject.next(null);
            }
        });
    }
}
