import { Subject, Subscription } from 'rxjs';
import { ReactiveApp } from './ReactiveApp';
export class SubjectScheduler extends Subscription {
    constructor(scheduler, defaultObserverOrNext, subject = new Subject()) {
        super();
        this.scheduler = scheduler;
        this.defaultObserverOrNext = defaultObserverOrNext;
        this.subject = subject;
        this.observerRefCount = 0;
        this.add(this.subject);
        if (this.defaultObserverOrNext != null) {
            this.defaultObserverSub = this.getScheduledObservable()
                .subscribe(defaultObserverOrNext, ReactiveApp.defaultErrorHandler.next);
        }
    }
    next(value) {
        this.subject.next(value);
    }
    error(err) {
        this.subject.error(err);
    }
    complete() {
        this.subject.complete();
    }
    getScheduledObservable() {
        return this.scheduler == null ?
            this.subject.asObservable() :
            this.subject.observeOn(this.scheduler);
    }
    subscribe(observerOrNext, error = ReactiveApp.defaultErrorHandler.next, complete) {
        if (this.defaultObserverSub) {
            this.defaultObserverSub.unsubscribe();
            this.defaultObserverSub = undefined;
        }
        ++this.observerRefCount;
        const sub = this.getScheduledObservable()
            .subscribe(observerOrNext, error, complete);
        sub.add(new Subscription(() => {
            if (--this.observerRefCount <= 0 && this.defaultObserverOrNext != null) {
                this.observerRefCount = 0;
                this.defaultObserverSub = this.getScheduledObservable()
                    .subscribe(this.defaultObserverOrNext, ReactiveApp.defaultErrorHandler.next);
            }
        }));
        return sub;
    }
}
